@@ -1,4 +1,4 @@
-# Copyright (C) 2016-2020  Vincent Pelletier <plr.vincent@gmail.com>
+# Copyright (C) 2016-2024  Vincent Pelletier <plr.vincent@gmail.com>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -85,6 +85,7 @@ from io import StringIO, BytesIO
 from importlib import import_module
 import itertools
 from collections import defaultdict
+import sys
 import pprofile
 
 if sys.version_info >= (3, ):
@@ -102,7 +103,7 @@ def getFuncCodeOrNone(module, attribute_path):
     return value
 
 DB_query_func_code = getFuncCodeOrNone('Products.ZMySQLDA.db', ('DB', '_query'))
-ZODB_setstate_func_code = getFuncCodeOrNone('ZODB.Connection', ('Connection', '_setstate'))
+ZODB_setstate_func_code = getFuncCodeOrNone('ZODB.Connection', ('Connection', 'setstate'))
 PythonExpr__call__func_code = getFuncCodeOrNone('zope.tales.pythonexpr', ('PythonExpr', '__call__'))
 ZRPythonExpr__call__func_code = getFuncCodeOrNone('Products.PageTemplates.ZRPythonExpr', ('PythonExpr', '__call__'))
 DT_UtilEvaleval_func_code = getFuncCodeOrNone('DocumentTemplate.DT_Util', ('Eval', 'eval'))
@@ -287,9 +288,18 @@ class ZopeMixIn(object):
             evaluator_frame = frame.f_back
             while evaluator_frame is not None:
                 evaluator_code = evaluator_frame.f_code
+                if evaluator_code is PythonScript_exec_func_code:
+                    if evaluator_frame.f_locals.get('safe_globals') is frame_globals:
+                        evaluated_module_unique = evaluator_frame.f_locals['function_code']
+                        break
+                    elif evaluator_frame.f_locals.get('g') is frame_globals:
+                        evaluated_module_unique = evaluator_frame.f_locals['fcode']
+                        break
                 if (
-                    evaluator_code is PythonScript_exec_func_code and
-                    evaluator_frame.f_locals.get('g') is frame_globals
+                    evaluator_code is PythonScript_exec_func_code and (
+                      evaluator_frame.f_locals.get('safe_globals') is frame_globals or
+                      evaluator_frame.f_locals.get('g') is frame_globals
+                    )
                 ):
                     evaluated_module_unique = evaluator_frame.f_locals['fcode']
                     break
